@@ -2,13 +2,13 @@ using System;
 using System.Reflection;
 using System.Runtime.CompilerServices;
 
-[assembly: ApplicationVersion ("1.5.2", "1.6 Beta 3")]
-[assembly: ApplicationBuildInformation ("git-checkout", "linux-gnu", "i686", "2009-11-20 19:17:59 EST")]
+[assembly: ApplicationVersion ("1.5.7", "1.6 RC3")]
+[assembly: ApplicationBuildInformation ("git-checkout", "linux-gnu", "i686", "2010-03-22 20:59:26 EDT")]
 
 [assembly: AssemblyVersion ("1.5.0.0")]
 [assembly: AssemblyTitle ("Banshee")]
 [assembly: AssemblyDescription ("Banshee Media Player")]
-[assembly: AssemblyCopyright ("Copyright (C) 2005-2009 Novell Inc. and others")]
+[assembly: AssemblyCopyright ("Copyright (C) 2005-2010 Novell Inc. and others")]
 [assembly: AssemblyCompany ("Novell, Inc.")]
 
 [AttributeUsage (AttributeTargets.Assembly, Inherited = false)]
