@@ -34,6 +34,7 @@ using Mono.Addins;
 using Hyena;
 
 using Banshee.Base;
+using Banshee.Metrics;
 using Banshee.Database;
 using Banshee.ServiceStack;
 using Banshee.Gui.Dialogs;
@@ -92,6 +93,8 @@ namespace Banshee.Gui
 
         protected virtual void InitializeGtk ()
         {
+            Log.Debug ("Initializing GTK");
+
             if (!GLib.Thread.Supported) {
                 GLib.Thread.Init ();
             }
@@ -110,7 +113,7 @@ namespace Banshee.Gui
         protected void Initialize (bool registerCommonServices)
         {
             // Set the process name so system process listings and commands are pretty
-            PlatformHacks.TrySetProcessName (Application.InternalName);
+            ApplicationContext.TrySetProcessName (Application.InternalName);
 
             Application.Initialize ();
 
@@ -119,12 +122,16 @@ namespace Banshee.Gui
             Gtk.Window.DefaultIconName = default_icon_name;
 
             ThreadAssist.InitializeMainThread ();
+            ThreadAssist.ProxyToMainHandler = Banshee.ServiceStack.Application.Invoke;
 
             Gdk.Global.ProgramClass = Application.InternalName;
+#if HAVE_GLIBSHARP_2_12_7
             GLib.Global.ApplicationName = "Banshee";
+#endif
 
             if (ApplicationContext.Debugging) {
                 GLib.Log.SetLogHandler ("Gtk", GLib.LogLevelFlags.Critical, GLib.Log.PrintTraceLogFunction);
+                Gdk.Window.DebugUpdates = !String.IsNullOrEmpty (Environment.GetEnvironmentVariable ("GDK_DEBUG_UPDATES"));
             }
 
             ServiceManager.ServiceStarted += OnServiceStarted;
@@ -140,6 +147,7 @@ namespace Banshee.Gui
             Application.TimeoutHandler = RunTimeout;
             Application.IdleHandler = RunIdle;
             Application.IdleTimeoutRemoveHandler = IdleTimeoutRemove;
+            BansheeMetrics.Started += OnMetricsStarted;
 
             // Start the core boot process
 
@@ -155,9 +163,22 @@ namespace Banshee.Gui
             Log.Notify += OnLogNotify;
         }
 
+        private void OnMetricsStarted ()
+        {
+            var metrics = BansheeMetrics.Instance;
+            var screen = Gdk.Screen.Default;
+
+            metrics.Add ("Display/NScreens",    Gdk.Display.Default.NScreens);
+            metrics.Add ("Screen/Height",       screen.Height);
+            metrics.Add ("Screen/Width",        screen.Width);
+            metrics.Add ("Screen/IsComposited", screen.IsComposited);
+            metrics.Add ("Screen/NMonitors",    screen.NMonitors);
+        }
+
         public virtual void Run ()
         {
             RunIdle (delegate { OnStarted (); return false; });
+            Log.Debug ("Starting GTK main loop");
             Gtk.Application.Run ();
         }
 
@@ -220,7 +241,7 @@ namespace Banshee.Gui
                     break;
             }
 
-            Banshee.Widgets.HigMessageDialog dialog = new Banshee.Widgets.HigMessageDialog (
+            Hyena.Widgets.HigMessageDialog dialog = new Hyena.Widgets.HigMessageDialog (
                 window, Gtk.DialogFlags.Modal, mtype, Gtk.ButtonsType.Close, entry.Message, entry.Details);
 
             dialog.Title = String.Empty;
